@@ -1,8 +1,10 @@
 use crate::{arc_ref::ArcRef, marker::*};
 use std::{
     any::Any,
+    borrow::Borrow,
     cmp, fmt,
     fmt::{Debug, Display},
+    hash::{Hash, Hasher},
     marker::PhantomData,
     ops::Deref,
     ptr,
@@ -48,12 +50,14 @@ where
     O: ?Sized,
     E: EqKind,
 {
+    /// Discard data and return owner in [Arc].
     pub fn into_arc(from: ArcOwned<'a, O, I, E>) -> Arc<O> {
         let Self { owner, inner, .. } = from;
         drop(inner);
         owner
     }
 
+    /// Reset data to reference to owner.
     pub fn into_owner_ref(this: ArcOwned<'a, O, I, E>) -> ArcOwned<'a, O, &O, E> {
         let Self { owner, inner, .. } = this;
         drop(inner);
@@ -70,18 +74,22 @@ where
         }
     }
 
+    /// Get reference to owner.
     pub fn owner(this: &'a ArcOwned<'a, O, I, E>) -> &'a O {
         &this.owner
     }
 
+    /// Get strong count on owner.
     pub fn strong_count(this: &ArcOwned<'a, O, I, E>) -> usize {
         Arc::strong_count(&this.owner)
     }
 
+    /// Get weak count on owner.
     pub fn weak_count(this: &ArcOwned<'a, O, I, E>) -> usize {
         Arc::weak_count(&this.owner)
     }
 
+    /// Applies function `f` to data.
     pub fn map<T, F>(self, f: F) -> ArcOwned<'a, O, T, E>
     where
         F: FnOnce(I) -> T,
@@ -95,6 +103,7 @@ where
         }
     }
 
+    /// Applies fallible function `f` to data.
     pub fn try_map<Ok, Err, F>(self, f: F) -> Result<ArcOwned<'a, O, Ok, E>, Err>
     where
         F: FnOnce(I) -> Result<Ok, Err>,
@@ -108,6 +117,7 @@ where
         })
     }
 
+    /// Applies function `f` that returns optional value to data.
     pub fn filter_map<T, F>(self, f: F) -> Option<ArcOwned<'a, O, T, E>>
     where
         F: FnOnce(I) -> Option<T>,
@@ -121,6 +131,7 @@ where
         })
     }
 
+    /// Flatten the wrapped iterable data into an iterator of wrapped items.
     pub fn flatten(self) -> impl Iterator<Item = ArcOwned<'a, O, I::Item, E>>
     where
         I: IntoIterator,
@@ -137,6 +148,7 @@ where
         })
     }
 
+    /// Apply fucntion `f` to get an iterable type, and flatten it to an iterator of wrapped items.
     pub fn flat_map<T, F>(self, f: F) -> impl Iterator<Item = ArcOwned<'a, O, T::Item, E>>
     where
         F: FnOnce(I) -> T,
@@ -150,6 +162,7 @@ impl<'a, O, I, E> ArcOwned<'a, O, I, E>
 where
     E: EqKind,
 {
+    /// Build from an owner.
     pub fn new(owner: O) -> Self
     where
         Self: From<Arc<O>>,
@@ -157,6 +170,7 @@ where
         Arc::new(owner).into()
     }
 
+    /// Change the owner type to [Any] trait object.
     pub fn into_any_owner(
         from: ArcOwned<'a, O, I, E>,
     ) -> ArcOwned<'a, dyn Any + Send + Sync + 'static, I, E>
@@ -172,14 +186,27 @@ where
         }
     }
 
-    pub fn try_unwrap_owner(from: ArcOwned<'a, O, I, E>) -> Option<O> {
+    /// Unwrap the owner if strong count is one.
+    pub fn try_unwrap_owner(from: ArcOwned<'a, O, I, E>) -> Result<O, Self> {
         let Self { owner, inner, .. } = from;
-        drop(inner);
-        Arc::try_unwrap(owner).ok()
+
+        match Arc::try_unwrap(owner) {
+            Ok(owner) => Ok(owner),
+            Err(owner) => Err(Self {
+                owner,
+                inner,
+                _phantom: PhantomData,
+            }),
+        }
     }
 
+    /// Unwrap the owner and panic if strong count is one.
+    ///
+    /// # Panic
+    /// The method panics if strong count is not 1.
     pub fn unwrap_owner(from: ArcOwned<'a, O, I, E>) -> O {
-        Self::try_unwrap_owner(from).unwrap()
+        Self::try_unwrap_owner(from)
+            .unwrap_or_else(|_| panic!("unable to unwrap because strong count is greater than 1"))
     }
 }
 
@@ -189,6 +216,7 @@ where
     I: ?Sized,
     E: EqKind,
 {
+    /// Convert ot [ArcRef].
     pub fn into_arc_ref(this: ArcOwned<'a, O, &'a I, E>) -> ArcRef<'a, O, I, E> {
         let Self { owner, inner, .. } = this;
 
@@ -205,6 +233,7 @@ where
     O: ?Sized,
     E: EqKind,
 {
+    /// Transpose an [ArcOwned] of an [Option] to an [Option] of an [ArcOwned].
     pub fn transpose(self) -> Option<ArcOwned<'a, O, I, E>> {
         let Self { owner, inner, .. } = self;
         Some(ArcOwned {
@@ -220,6 +249,7 @@ where
     O: ?Sized,
     E: EqKind,
 {
+    /// Transpose an [ArcOwned] of a [Result] to a [Result] of an [ArcOwned].
     pub fn transpose(self) -> Result<ArcOwned<'a, O, Ok, E>, Err> {
         let Self { owner, inner, .. } = self;
         Ok(ArcOwned {
@@ -234,9 +264,8 @@ impl<'a, I, E> ArcOwned<'a, dyn Any + Send + Sync + 'static, I, E>
 where
     E: EqKind,
 {
-    pub fn downcast_owner<O>(
-        this: ArcOwned<'a, dyn Any + Send + Sync + 'static, I, E>,
-    ) -> Result<ArcOwned<'a, O, I, E>, ArcOwned<'a, dyn Any + Send + Sync + 'static, I, E>>
+    /// Downcast the [Any]-trait object owner to concrete type.
+    pub fn downcast_owner<O>(this: Self) -> Result<ArcOwned<'a, O, I, E>, Self>
     where
         O: Send + Sync + 'static,
     {
@@ -263,6 +292,7 @@ where
     I: Clone,
     E: EqKind,
 {
+    /// Clone the data and increase reference count to owner.
     fn clone(&self) -> Self {
         let Self { owner, inner, .. } = self;
 
@@ -333,6 +363,19 @@ where
     }
 }
 
+impl<'a, O, I> Hash for ArcOwned<'a, O, I, ByContent>
+where
+    O: ?Sized,
+    I: Hash,
+{
+    fn hash<H>(&self, state: &mut H)
+    where
+        H: Hasher,
+    {
+        self.inner.hash(state);
+    }
+}
+
 impl<'a, O, I> PartialEq<Self> for ArcOwned<'a, O, &'a I, ByAddress>
 where
     O: ?Sized,
@@ -370,6 +413,18 @@ where
     }
 }
 
+impl<'a, O, I> Hash for ArcOwned<'a, O, &'a I, ByAddress>
+where
+    O: ?Sized,
+{
+    fn hash<H>(&self, state: &mut H)
+    where
+        H: Hasher,
+    {
+        ptr::hash(self.inner as *const I, state);
+    }
+}
+
 impl<'a, O, I, E> AsRef<I> for ArcOwned<'a, O, I, E>
 where
     O: ?Sized,
@@ -380,6 +435,16 @@ where
     }
 }
 
+impl<'a, O, I, E> Borrow<I> for ArcOwned<'a, O, I, E>
+where
+    O: ?Sized,
+    E: EqKind,
+{
+    fn borrow(&self) -> &I {
+        self.deref()
+    }
+}
+
 impl<'a, O, I, E> Deref for ArcOwned<'a, O, I, E>
 where
     O: ?Sized,
