@@ -1,8 +1,10 @@
 use crate::{arc_owned::ArcOwned, marker::*};
 use std::{
     any::Any,
+    borrow::Borrow,
     cmp, fmt,
     fmt::{Debug, Display},
+    hash::{Hash, Hasher},
     marker::PhantomData,
     ops::Deref,
     ptr,
@@ -39,6 +41,7 @@ where
     O: ?Sized,
     E: EqKind,
 {
+    /// Build from owner data in [Arc].
     pub fn from_arc(owner: Arc<O>) -> Self {
         owner.into()
     }
@@ -50,11 +53,13 @@ where
     I: ?Sized,
     E: EqKind,
 {
+    /// Discard the inner reference and return the owner in [Arc].
     pub fn into_arc(from: ArcRef<'a, O, I, E>) -> Arc<O> {
         let Self { owner, .. } = from;
         owner
     }
 
+    /// Convert to [ArcOwned].
     pub fn into_arc_owned(this: ArcRef<'a, O, I, E>) -> ArcOwned<'a, O, &'a I, E> {
         let Self { owner, inner, .. } = this;
         ArcOwned {
@@ -64,6 +69,7 @@ where
         }
     }
 
+    /// Reset the inner reference to the owner.
     pub fn into_owner_ref(this: ArcRef<'a, O, I, E>) -> ArcRef<'a, O, O, E> {
         let Self { owner, .. } = this;
 
@@ -79,18 +85,22 @@ where
         }
     }
 
+    /// Get the reference to the owner.
     pub fn owner(this: &'a ArcRef<'a, O, I, E>) -> &'a O {
         &this.owner
     }
 
+    /// Get the strong count on the owner.
     pub fn strong_count(this: &ArcRef<'a, O, I, E>) -> usize {
         Arc::strong_count(&this.owner)
     }
 
+    /// Get the weak count on the owner.
     pub fn weak_count(this: &ArcRef<'a, O, I, E>) -> usize {
         Arc::weak_count(&this.owner)
     }
 
+    /// Apply function `f` to the inner reference.
     pub fn map<T, F>(self, f: F) -> ArcRef<'a, O, T, E>
     where
         F: FnOnce(&'a I) -> &'a T,
@@ -105,6 +115,7 @@ where
         }
     }
 
+    /// Apply fallible function `f` to the inner reference.
     pub fn try_map<Ok, Err, F>(self, f: F) -> Result<ArcRef<'a, O, Ok, E>, Err>
     where
         F: FnOnce(&'a I) -> Result<&'a Ok, Err>,
@@ -119,6 +130,7 @@ where
         })
     }
 
+    /// Apply function `f` that returns an optional reference to the inner reference.
     pub fn filter_map<T, F>(self, f: F) -> Option<ArcRef<'a, O, T, E>>
     where
         F: FnOnce(&'a I) -> Option<&'a T>,
@@ -133,6 +145,7 @@ where
         })
     }
 
+    /// Flatten the wrapped iterable inner reference into an iterator of wrapped items.
     pub fn flatten<T>(self) -> impl Iterator<Item = ArcRef<'a, O, T, E>>
     where
         &'a I: IntoIterator<Item = &'a T>,
@@ -150,6 +163,7 @@ where
         })
     }
 
+    /// Apply fucntion `f` to get an iterable type, and flatten it to an iterator of references.
     pub fn flat_map<T, C, F>(self, f: F) -> impl Iterator<Item = ArcRef<'a, O, T, E>>
     where
         F: FnOnce(&'a I) -> C,
@@ -175,6 +189,7 @@ impl<'a, O, I, E> ArcRef<'a, O, I, E>
 where
     E: EqKind,
 {
+    /// Build from an owner.
     pub fn new(owner: O) -> Self
     where
         Self: From<Arc<O>>,
@@ -182,6 +197,7 @@ where
         Arc::new(owner).into()
     }
 
+    /// Convert the owner type to [Any] trait object.
     pub fn into_any_owner(
         from: ArcRef<'a, O, I, E>,
     ) -> ArcRef<'a, dyn Any + Send + Sync + 'static, I, E>
@@ -197,13 +213,27 @@ where
         }
     }
 
-    pub fn try_unwrap_owner(from: ArcRef<'a, O, I, E>) -> Option<O> {
-        let Self { owner, .. } = from;
-        Arc::try_unwrap(owner).ok()
+    /// Unwrap the owner if strong count is one.
+    pub fn try_unwrap_owner(from: ArcRef<'a, O, I, E>) -> Result<O, Self> {
+        let Self { owner, inner, .. } = from;
+
+        match Arc::try_unwrap(owner) {
+            Ok(owner) => Ok(owner),
+            Err(owner) => Err(Self {
+                owner,
+                inner,
+                _phantom: PhantomData,
+            }),
+        }
     }
 
+    /// Unwrap the owner and panic if strong count is one.
+    ///
+    /// # Panic
+    /// The method panics if strong count is not 1.
     pub fn unwrap_owner(from: ArcRef<'a, O, I, E>) -> O {
-        Self::try_unwrap_owner(from).unwrap()
+        Self::try_unwrap_owner(from)
+            .unwrap_or_else(|_| panic!("unable to unwrap because strong count is greater than 1"))
     }
 }
 
@@ -212,9 +242,8 @@ where
     I: ?Sized,
     E: EqKind,
 {
-    pub fn downcast_owner<O>(
-        this: ArcRef<'a, dyn Any + Send + Sync + 'static, I, E>,
-    ) -> Result<ArcRef<'a, O, I, E>, ArcRef<'a, dyn Any + Send + Sync + 'static, I, E>>
+    /// Downcast the [Any]-trait object owner to concrete type.
+    pub fn downcast_owner<O>(this: Self) -> Result<ArcRef<'a, O, I, E>, Self>
     where
         O: Send + Sync + 'static,
     {
@@ -241,6 +270,7 @@ where
     I: ?Sized,
     E: EqKind,
 {
+    /// Copy the inner reference and increase reference count to owner.
     fn clone(&self) -> Self {
         let Self { owner, inner, .. } = self;
 
@@ -317,6 +347,19 @@ where
     }
 }
 
+impl<'a, O, I> Hash for ArcRef<'a, O, I, ByContent>
+where
+    O: ?Sized,
+    I: ?Sized + Hash,
+{
+    fn hash<H>(&self, state: &mut H)
+    where
+        H: Hasher,
+    {
+        self.inner.hash(state);
+    }
+}
+
 impl<'a, O, I> PartialEq<Self> for ArcRef<'a, O, I, ByAddress>
 where
     O: ?Sized,
@@ -354,6 +397,19 @@ where
     }
 }
 
+impl<'a, O, I> Hash for ArcRef<'a, O, I, ByAddress>
+where
+    O: ?Sized,
+    I: ?Sized,
+{
+    fn hash<H>(&self, state: &mut H)
+    where
+        H: Hasher,
+    {
+        ptr::hash(self.inner as *const I, state);
+    }
+}
+
 impl<'a, O, I, E> AsRef<I> for ArcRef<'a, O, I, E>
 where
     O: ?Sized,
@@ -365,6 +421,17 @@ where
     }
 }
 
+impl<'a, O, I, E> Borrow<I> for ArcRef<'a, O, I, E>
+where
+    O: ?Sized,
+    I: ?Sized,
+    E: EqKind,
+{
+    fn borrow(&self) -> &I {
+        self.deref()
+    }
+}
+
 impl<'a, O, I, E> Deref for ArcRef<'a, O, I, E>
 where
     O: ?Sized,
