@@ -1,8 +1,10 @@
 use crate::{arc_owned::ArcOwned, arc_ref::ArcRef, box_ref::BoxRef, marker::*};
 use std::{
     any::Any,
+    borrow::Borrow,
     cmp, fmt,
     fmt::{Debug, Display},
+    hash::{Hash, Hasher},
     marker::PhantomData,
     ops::{Deref, DerefMut},
     ptr,
@@ -43,6 +45,7 @@ where
     O: ?Sized,
     E: EqKind,
 {
+    /// Build from boxed data.
     pub fn from_box(owner: Box<O>) -> Self {
         owner.into()
     }
@@ -53,12 +56,14 @@ where
     O: ?Sized,
     E: EqKind,
 {
+    /// Discard owned data and return boxed owner.
     pub fn into_box(from: BoxOwned<'a, O, I, E>) -> Box<O> {
         let Self { owner, inner, .. } = from;
         drop(inner);
         owner
     }
 
+    /// Convert to [ArcOwned] without re-allocation.
     pub fn into_arc_owned(from: BoxOwned<'a, O, I, E>) -> ArcOwned<'a, O, I, E> {
         let Self { owner, inner, .. } = from;
         ArcOwned {
@@ -68,6 +73,7 @@ where
         }
     }
 
+    /// Reset data to the reference to owner.
     pub fn into_owner_ref(this: BoxOwned<'a, O, I, E>) -> BoxOwned<'a, O, &mut O, E> {
         let Self {
             mut owner, inner, ..
@@ -86,10 +92,12 @@ where
         }
     }
 
+    /// Get the reference to owner.
     pub fn owner(this: &'a BoxOwned<'a, O, I, E>) -> &'a O {
         &this.owner
     }
 
+    /// Applies function `f` to data.
     pub fn map<T, F>(self, f: F) -> BoxOwned<'a, O, T, E>
     where
         F: FnOnce(I) -> T,
@@ -103,6 +111,7 @@ where
         }
     }
 
+    /// Applies fallible function `f` to data.
     pub fn try_map<Ok, Err, F>(self, f: F) -> Result<BoxOwned<'a, O, Ok, E>, Err>
     where
         F: FnOnce(I) -> Result<Ok, Err>,
@@ -116,6 +125,7 @@ where
         })
     }
 
+    /// Applies function `f` that returns optional value to data.
     pub fn filter_map<T, F>(self, f: F) -> Option<BoxOwned<'a, O, T, E>>
     where
         F: FnOnce(I) -> Option<T>,
@@ -134,6 +144,7 @@ impl<'a, O, I, E> BoxOwned<'a, O, I, E>
 where
     E: EqKind,
 {
+    /// Build from an owner.
     pub fn new(owner: O) -> Self
     where
         Self: From<Box<O>>,
@@ -141,12 +152,14 @@ where
         Box::new(owner).into()
     }
 
+    /// Discard the data and return the owner.
     pub fn into_owner(from: BoxOwned<'a, O, I, E>) -> O {
         let Self { owner, inner, .. } = from;
         drop(inner);
         *owner
     }
 
+    /// Change the owner type to [Any] + [Send] trait object.
     pub fn into_any_owner(
         from: BoxOwned<'a, O, I, E>,
     ) -> BoxOwned<'a, dyn Any + Send + 'static, I, E>
@@ -162,6 +175,7 @@ where
         }
     }
 
+    /// Change the owner type to [Any] trait object.
     pub fn into_any_owner_local(
         from: BoxOwned<'a, O, I, E>,
     ) -> BoxOwned<'a, dyn Any + 'static, I, E>
@@ -184,6 +198,7 @@ where
     I: ?Sized,
     E: EqKind,
 {
+    /// Convert to [BoxRef].
     pub fn into_box_ref(self) -> BoxRef<'a, O, I, E> {
         let Self { owner, inner, .. } = self;
 
@@ -194,6 +209,7 @@ where
         }
     }
 
+    /// Convert to [ArcRef].
     pub fn into_arc_ref(self) -> ArcRef<'a, O, I, E> {
         let Self { owner, inner, .. } = self;
 
@@ -211,6 +227,7 @@ where
     I: ?Sized,
     E: EqKind,
 {
+    /// Convert to [ArcRef].
     pub fn into_arc_ref(self) -> ArcRef<'a, O, I, E> {
         let Self { owner, inner, .. } = self;
 
@@ -227,6 +244,7 @@ where
     O: ?Sized,
     E: EqKind,
 {
+    /// Transpose an [BoxOwned] of a [Option] to a [Option] of an [BoxOwned].
     pub fn transpose(self) -> Option<BoxOwned<'a, O, I, E>> {
         let Self { owner, inner, .. } = self;
         Some(BoxOwned {
@@ -242,6 +260,7 @@ where
     O: ?Sized,
     E: EqKind,
 {
+    /// Transpose an [BoxOwned] of a [Result] to a [Result] of an [BoxOwned].
     pub fn transpose(self) -> Result<BoxOwned<'a, O, Ok, E>, Err> {
         let Self { owner, inner, .. } = self;
         Ok(BoxOwned {
@@ -256,9 +275,8 @@ impl<'a, I, E> BoxOwned<'a, dyn Any + Send + 'static, I, E>
 where
     E: EqKind,
 {
-    pub fn downcast_owner<O>(
-        this: BoxOwned<'a, dyn Any + Send + 'static, I, E>,
-    ) -> Result<BoxOwned<'a, O, I, E>, BoxOwned<'a, dyn Any + Send + 'static, I, E>>
+    /// Downcast the [Any] + [Send] trait object owner to concrete type.
+    pub fn downcast_owner<O>(this: Self) -> Result<BoxOwned<'a, O, I, E>, Self>
     where
         O: Send + 'static,
     {
@@ -283,9 +301,8 @@ impl<'a, I, E> BoxOwned<'a, dyn Any + 'static, I, E>
 where
     E: EqKind,
 {
-    pub fn downcast_owner_local<O>(
-        this: BoxOwned<'a, dyn Any + 'static, I, E>,
-    ) -> Result<BoxOwned<'a, O, I, E>, BoxOwned<'a, dyn Any + 'static, I, E>>
+    /// Downcast the [Any] trait object owner to concrete type.
+    pub fn downcast_owner_local<O>(this: Self) -> Result<BoxOwned<'a, O, I, E>, Self>
     where
         O: 'static,
     {
@@ -365,6 +382,19 @@ where
     }
 }
 
+impl<'a, O, I> Hash for BoxOwned<'a, O, I, ByContent>
+where
+    O: ?Sized,
+    I: Hash,
+{
+    fn hash<H>(&self, state: &mut H)
+    where
+        H: Hasher,
+    {
+        self.inner.hash(state);
+    }
+}
+
 impl<'a, O, I> PartialEq<Self> for BoxOwned<'a, O, &'a mut I, ByAddress>
 where
     O: ?Sized,
@@ -402,6 +432,19 @@ where
     }
 }
 
+impl<'a, O, I> Hash for BoxOwned<'a, O, &'a mut I, ByAddress>
+where
+    O: ?Sized,
+    I: ?Sized,
+{
+    fn hash<H>(&self, state: &mut H)
+    where
+        H: Hasher,
+    {
+        ptr::hash(self.inner as *const I, state);
+    }
+}
+
 impl<'a, O, I> PartialEq<Self> for BoxOwned<'a, O, &'a I, ByAddress>
 where
     O: ?Sized,
@@ -439,6 +482,19 @@ where
     }
 }
 
+impl<'a, O, I> Hash for BoxOwned<'a, O, &'a I, ByAddress>
+where
+    O: ?Sized,
+    I: ?Sized,
+{
+    fn hash<H>(&self, state: &mut H)
+    where
+        H: Hasher,
+    {
+        ptr::hash(self.inner as *const I, state);
+    }
+}
+
 impl<'a, O, I, E> AsRef<I> for BoxOwned<'a, O, I, E>
 where
     O: ?Sized,
@@ -459,6 +515,16 @@ where
     }
 }
 
+impl<'a, O, I, E> Borrow<I> for BoxOwned<'a, O, I, E>
+where
+    O: ?Sized,
+    E: EqKind,
+{
+    fn borrow(&self) -> &I {
+        self.deref()
+    }
+}
+
 impl<'a, O, I, E> Deref for BoxOwned<'a, O, I, E>
 where
     O: ?Sized,
