@@ -1,8 +1,10 @@
 use crate::{arc_owned::ArcOwned, arc_ref::ArcRef, box_owned::BoxOwned, marker::*};
 use std::{
     any::Any,
+    borrow::Borrow,
     cmp, fmt,
     fmt::{Debug, Display},
+    hash::{Hash, Hasher},
     marker::PhantomData,
     ops::{Deref, DerefMut},
     ptr,
@@ -44,6 +46,7 @@ where
     O: ?Sized,
     E: EqKind,
 {
+    /// Build from boxed data.
     pub fn from_box(owner: Box<O>) -> Self {
         owner.into()
     }
@@ -53,6 +56,7 @@ impl<'a, O, E> BoxRef<'a, O, O, E>
 where
     E: EqKind,
 {
+    /// Build from owner.
     pub fn new(owner: O) -> Self {
         Box::new(owner).into()
     }
@@ -64,11 +68,13 @@ where
     I: ?Sized,
     E: EqKind,
 {
+    /// Discard the inner reference and return boxed owner.
     pub fn into_box(from: BoxRef<'a, O, I, E>) -> Box<O> {
         let Self { owner, .. } = from;
         owner
     }
 
+    /// Convert to [BoxOwned].
     pub fn into_box_owned(from: BoxRef<'a, O, I, E>) -> BoxOwned<'a, O, &mut I, E> {
         let Self { owner, inner, .. } = from;
         BoxOwned {
@@ -78,6 +84,7 @@ where
         }
     }
 
+    /// Convert to [ArcOwned] without re-allocation.
     pub fn into_arc_owned(from: BoxRef<'a, O, I, E>) -> ArcOwned<'a, O, &'a mut I, E> {
         let Self { owner, inner, .. } = from;
         ArcOwned {
@@ -87,6 +94,7 @@ where
         }
     }
 
+    /// Convert to [ArcRef] without re-allocation.
     pub fn into_arc_ref(from: BoxRef<'a, O, I, E>) -> ArcRef<'a, O, I, E> {
         let Self { owner, inner, .. } = from;
         ArcRef {
@@ -96,6 +104,7 @@ where
         }
     }
 
+    /// Reset the inner reference to the reference to owner.
     pub fn into_owner_ref(this: BoxRef<'a, O, I, E>) -> BoxRef<'a, O, O, E> {
         let Self { mut owner, .. } = this;
 
@@ -111,10 +120,12 @@ where
         }
     }
 
+    /// Get the reference to the owner.
     pub fn owner(this: &'a BoxRef<'a, O, I, E>) -> &'a O {
         &this.owner
     }
 
+    /// Applies function `f` to inner reference.
     pub fn map<T, F>(self, f: F) -> BoxRef<'a, O, T, E>
     where
         F: FnOnce(&'a mut I) -> &'a mut T,
@@ -129,6 +140,7 @@ where
         }
     }
 
+    /// Applies fallible function `f` to inner reference.
     pub fn try_map<Ok, Err, F>(self, f: F) -> Result<BoxRef<'a, O, Ok, E>, Err>
     where
         F: FnOnce(&'a mut I) -> Result<&'a mut Ok, Err>,
@@ -143,6 +155,7 @@ where
         })
     }
 
+    /// Applies function `f` that returns optional reference to inner reference.
     pub fn filter_map<T, F>(self, f: F) -> Option<BoxRef<'a, O, T, E>>
     where
         F: FnOnce(&'a mut I) -> Option<&'a mut T>,
@@ -162,11 +175,13 @@ impl<'a, O, I, E> BoxRef<'a, O, I, E>
 where
     E: EqKind,
 {
+    /// Discard the inner reference and return the owner.
     pub fn into_owner(from: BoxRef<'a, O, I, E>) -> O {
         let Self { owner, .. } = from;
         *owner
     }
 
+    /// Change the owner type to [Any] + [Send] trait object.
     pub fn into_any_owner(from: BoxRef<'a, O, I, E>) -> BoxRef<'a, dyn Any + Send + 'static, I, E>
     where
         O: Send + 'static,
@@ -180,6 +195,7 @@ where
         }
     }
 
+    /// Change the owner type to [Any] trait object.
     pub fn into_any_owner_local(from: BoxRef<'a, O, I, E>) -> BoxRef<'a, dyn Any + 'static, I, E>
     where
         O: 'static,
@@ -199,9 +215,8 @@ where
     I: ?Sized,
     E: EqKind,
 {
-    pub fn downcast_owner<O>(
-        this: BoxRef<'a, dyn Any + Send + 'static, I, E>,
-    ) -> Result<BoxRef<'a, O, I, E>, BoxRef<'a, dyn Any + Send + 'static, I, E>>
+    /// Downcast the [Any] + [Send] trait object owner to concrete type.
+    pub fn downcast_owner<O>(this: Self) -> Result<BoxRef<'a, O, I, E>, Self>
     where
         O: Send + 'static,
     {
@@ -227,9 +242,8 @@ where
     I: ?Sized,
     E: EqKind,
 {
-    pub fn downcast_owner_local<O>(
-        this: BoxRef<'a, dyn Any + 'static, I, E>,
-    ) -> Result<BoxRef<'a, O, I, E>, BoxRef<'a, dyn Any + 'static, I, E>>
+    /// Downcast the [Any] trait object owner to concrete type.
+    pub fn downcast_owner_local<O>(this: Self) -> Result<BoxRef<'a, O, I, E>, Self>
     where
         O: 'static,
     {
@@ -315,6 +329,19 @@ where
     }
 }
 
+impl<'a, O, I> Hash for BoxRef<'a, O, I, ByContent>
+where
+    O: ?Sized,
+    I: ?Sized + Hash,
+{
+    fn hash<H>(&self, state: &mut H)
+    where
+        H: Hasher,
+    {
+        self.inner.hash(state);
+    }
+}
+
 impl<'a, O, I> PartialEq<Self> for BoxRef<'a, O, I, ByAddress>
 where
     O: ?Sized,
@@ -352,6 +379,19 @@ where
     }
 }
 
+impl<'a, O, I> Hash for BoxRef<'a, O, I, ByAddress>
+where
+    O: ?Sized,
+    I: ?Sized,
+{
+    fn hash<H>(&self, state: &mut H)
+    where
+        H: Hasher,
+    {
+        ptr::hash(self.inner as *const I, state);
+    }
+}
+
 impl<'a, O, I, E> AsRef<I> for BoxRef<'a, O, I, E>
 where
     O: ?Sized,
@@ -374,6 +414,17 @@ where
     }
 }
 
+impl<'a, O, I, E> Borrow<I> for BoxRef<'a, O, I, E>
+where
+    O: ?Sized,
+    I: ?Sized,
+    E: EqKind,
+{
+    fn borrow(&self) -> &I {
+        self.deref()
+    }
+}
+
 impl<'a, O, I, E> Deref for BoxRef<'a, O, I, E>
 where
     O: ?Sized,
